import base64
import io
import json
import struct
from json.decoder import JSONDecodeError
from typing import List

import numpy as np
from PIL import Image

from instill.helpers.const import (
    ImageToImageInput,
    TextGenerationChatInput,
    TextGenerationInput,
    TextToImageInput,
    VisualQuestionAnsweringInput,
)


def serialize_byte_tensor(input_tensor):
    """
    Serializes a bytes tensor into a flat numpy array of length prepended
    bytes. The numpy array should use dtype of np.object_. For np.bytes_,
    numpy will remove trailing zeros at the end of byte sequence and because
    of this it should be avoided.
    Parameters
    ----------
    input_tensor : np.array
        The bytes tensor to serialize.
    Returns
    -------
    serialized_bytes_tensor : np.array
        The 1-D numpy array of type uint8 containing the serialized bytes in 'C' order.
    Raises
    ------
    InferenceServerException
        If unable to serialize the given tensor.
    """

    if input_tensor.size == 0:
        return ()

    # If the input is a tensor of string/bytes objects, then must flatten those
    # into a 1-dimensional array containing the 4-byte byte size followed by the
    # actual element bytes. All elements are concatenated together in "C" order.
    if (input_tensor.dtype == np.object_) or (input_tensor.dtype.type == np.bytes_):
        flattened_ls: list = []
        for obj in np.nditer(input_tensor, flags=["refs_ok"], order="C"):
            # If directly passing bytes to BYTES type,
            # don't convert it to str as Python will encode the
            # bytes which may distort the meaning
            assert isinstance(obj, np.ndarray)
            if input_tensor.dtype == np.object_:
                if isinstance(obj.item(), bytes):
                    s = obj.item()
                else:
                    s = str(obj.item()).encode("utf-8")
            else:
                s = obj.item()
            flattened_ls.append(struct.pack("<I", len(s)))
            flattened_ls.append(s)
        flattened = b"".join(flattened_ls)
        return flattened
    return None


def deserialize_bytes_tensor(encoded_tensor):
    """
    Deserializes an encoded bytes tensor into an
    numpy array of dtype of python objects

    Parameters
    ----------
    encoded_tensor : bytes
        The encoded bytes tensor where each element
        has its length in first 4 bytes followed by
        the content
    Returns
    -------
    string_tensor : np.array
        The 1-D numpy array of type object containing the
        deserialized bytes in 'C' order.

    """
    strs = []
    offset = 0
    val_buf = encoded_tensor
    while offset < len(val_buf):
        l = struct.unpack_from("<I", val_buf, offset)[0]
        offset += 4
        sb = struct.unpack_from(f"<{l}s", val_buf, offset)[0]
        offset += l
        strs.append(sb)
    return np.array(strs, dtype=bytes)


class StandardTaskIO:
    @staticmethod
    def parse_task_text_generation_input(request) -> TextGenerationInput:
        text_generation_input = TextGenerationInput()

        for i, b_input_tensor in zip(request.inputs, request.raw_input_contents):
            input_name = i.name

            if input_name == "prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_generation_input.prompt = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `prompt` type\
                        ({type(text_generation_input.prompt)}): {text_generation_input.prompt}"
                )

            if input_name == "prompt_images":
                input_tensors = deserialize_bytes_tensor(b_input_tensor)
                images = []
                for enc in input_tensors:
                    if len(enc) == 0:
                        continue
                    try:
                        enc_json = json.loads(str(enc.decode("utf-8")))
                        if len(enc_json) == 0:
                            continue
                        decoded_enc = enc_json[0]
                    except JSONDecodeError:
                        print("[DEBUG] WARNING `enc_json` parsing faield!")
                    # pil_img = Image.open(io.BytesIO(enc.astype(bytes)))  # RGB
                    pil_img = Image.open(io.BytesIO(base64.b64decode(decoded_enc)))

                    image = np.array(pil_img)
                    if len(image.shape) == 2:  # gray image
                        raise ValueError(
                            f"The image shape with {image.shape} is "
                            f"not in acceptable"
                        )
                    images.append(image)
                # TODO: check wethere there are issues in batch size dimention
                text_generation_input.prompt_images = images
                print(
                    "[DEBUG] input `prompt_images` type"
                    f"({type(text_generation_input.prompt_images)}): "
                    f"{text_generation_input.prompt_images}"
                )

            if input_name == "chat_history":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                chat_history_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `chat_history_str` type"
                    f"({type(chat_history_str)}): "
                    f"{chat_history_str}"
                )
                try:
                    text_generation_input.chat_history = json.loads(chat_history_str)
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

            if input_name == "system_message":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_generation_input.system_message = str(
                    input_tensor[0].decode("utf-8")
                )
                print(
                    "[DEBUG] input `system_message` type"
                    f"({type(text_generation_input.system_message)}): "
                    f"{text_generation_input.system_message}"
                )

            if input_name == "max_new_tokens":
                text_generation_input.max_new_tokens = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `max_new_tokens` type"
                    f"({type(text_generation_input.max_new_tokens)}): "
                    f"{text_generation_input.max_new_tokens}"
                )

            if input_name == "top_k":
                text_generation_input.top_k = int.from_bytes(b_input_tensor, "little")
                print(
                    "[DEBUG] input `top_k` type"
                    f"({type(text_generation_input.top_k)}): "
                    f"{text_generation_input.top_k}"
                )

            if input_name == "temperature":
                text_generation_input.temperature = struct.unpack("f", b_input_tensor)[
                    0
                ]
                print(
                    "[DEBUG] input `temperature` type"
                    f"({type(text_generation_input.temperature)}): "
                    f"{text_generation_input.temperature}"
                )
                text_generation_input.temperature = round(
                    text_generation_input.temperature, 2
                )

            if input_name == "random_seed":
                text_generation_input.random_seed = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `random_seed` type"
                    f"({type(text_generation_input.random_seed)}): "
                    f"{text_generation_input.random_seed}"
                )

            if input_name == "extra_params":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                extra_params_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `extra_params` type"
                    f"({type(extra_params_str)}): "
                    f"{extra_params_str}"
                )

                try:
                    text_generation_input.extra_params = json.loads(extra_params_str)
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

        return text_generation_input

    @staticmethod
    def parse_task_text_generation_output(sequences: list):
        text_outputs = [seq["generated_text"].encode("utf-8") for seq in sequences]

        return serialize_byte_tensor(np.asarray(text_outputs))

    @staticmethod
    def parse_task_text_to_image_input(request) -> TextToImageInput:
        text_to_image_input = TextToImageInput()

        for i, b_input_tensor in zip(request.inputs, request.raw_input_contents):
            input_name = i.name

            if input_name == "prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_to_image_input.prompt = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `prompt` type\
                        ({type(text_to_image_input.prompt)}): {text_to_image_input.prompt}"
                )

            if input_name == "negative_prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_to_image_input.negative_prompt = str(
                    input_tensor[0].decode("utf-8")
                )
                print(
                    f"[DEBUG] input `negative_prompt` type\
                        ({type(text_to_image_input.negative_prompt)}): {text_to_image_input.negative_prompt}"
                )

            if input_name == "steps":
                text_to_image_input.steps = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `steps` type\
                        ({type(text_to_image_input.steps)}): {text_to_image_input.steps}"
                )

            if input_name == "seed":
                text_to_image_input.seed = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `seed` type\
                        ({type(text_to_image_input.seed)}): {text_to_image_input.seed}"
                )

            if input_name == "guidance_scale":
                text_to_image_input.guidance_scale = struct.unpack("f", b_input_tensor)[
                    0
                ]
                print(
                    f"[DEBUG] input `guidance_scale` type\
                        ({type(text_to_image_input.guidance_scale)}): {text_to_image_input.guidance_scale}"
                )
                text_to_image_input.guidance_scale = round(
                    text_to_image_input.guidance_scale, 2
                )

            if input_name == "samples":
                text_to_image_input.samples = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `samples` type\
                        ({type(text_to_image_input.samples)}): {text_to_image_input.samples}"
                )

            if input_name == "extra_params":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                extra_params_str = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `extra_params` type\
                        ({type(extra_params_str)}): {extra_params_str}"
                )

                try:
                    text_to_image_input.extra_params = json.loads(extra_params_str)
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

        return text_to_image_input

    @staticmethod
    def parse_task_text_to_image_output(image):
        return np.asarray(image).tobytes()

    @staticmethod
    def parse_task_image_to_image_input(request) -> ImageToImageInput:
        image_to_image_input = ImageToImageInput()

        for i, b_input_tensor in zip(request.inputs, request.raw_input_contents):
            input_name = i.name

            if input_name == "prompt_image":
                input_tensors = deserialize_bytes_tensor(b_input_tensor)
                images = []
                for enc in input_tensors:
                    pil_img = Image.open(io.BytesIO(enc.astype(bytes)))  # RGB
                    image = np.array(pil_img)
                    if len(image.shape) == 2:  # gray image
                        raise ValueError(
                            f"The image shape with {image.shape} is "
                            f"not in acceptable"
                        )
                    images.append(image)
                image_to_image_input.prompt_image = images[0]
                print(
                    f"[DEBUG] input `prompt_image` type\
                        ({type(image_to_image_input.prompt_image)}): {image_to_image_input.prompt_image}"
                )

            if input_name == "prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                image_to_image_input.prompt = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `prompt` type\
                        ({type(image_to_image_input.prompt)}): {image_to_image_input.prompt}"
                )

            if input_name == "steps":
                image_to_image_input.steps = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `steps` type\
                        ({type(image_to_image_input.steps)}): {image_to_image_input.steps}"
                )

            if input_name == "seed":
                image_to_image_input.seed = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `seed` type\
                        ({type(image_to_image_input.seed)}): {image_to_image_input.seed}"
                )

            if input_name == "guidance_scale":
                image_to_image_input.guidance_scale = struct.unpack(
                    "f", b_input_tensor
                )[0]
                print(
                    f"[DEBUG] input `guidance_scale` type\
                        ({type(image_to_image_input.guidance_scale)}): {image_to_image_input.guidance_scale}"
                )
                image_to_image_input.guidance_scale = round(
                    image_to_image_input.guidance_scale, 2
                )

            if input_name == "samples":
                image_to_image_input.samples = int.from_bytes(b_input_tensor, "little")
                print(
                    f"[DEBUG] input `samples` type\
                        ({type(image_to_image_input.samples)}): {image_to_image_input.samples}"
                )

            if input_name == "extra_params":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                extra_params_str = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `extra_params` type\
                        ({type(extra_params_str)}): {extra_params_str}"
                )

                try:
                    image_to_image_input.extra_params = json.loads(extra_params_str)
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

        return image_to_image_input

    @staticmethod
    def parse_task_image_to_image_output(image):
        return np.asarray(image).tobytes()

    @staticmethod
    def parse_task_text_generation_chat_input(request) -> TextGenerationChatInput:
        text_generation_chat_input = TextGenerationChatInput()

        for i, b_input_tensor in zip(request.inputs, request.raw_input_contents):
            input_name = i.name

            if input_name == "prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_generation_chat_input.prompt = str(input_tensor[0].decode("utf-8"))
                print(
                    f"[DEBUG] input `prompt` type\
                        ({type(text_generation_chat_input.prompt)}): {text_generation_chat_input.prompt}"
                )

            if input_name == "prompt_images":
                input_tensors = deserialize_bytes_tensor(b_input_tensor)
                images = []
                for enc in input_tensors:
                    if len(enc) == 0:
                        continue
                    try:
                        enc_json = json.loads(str(enc.decode("utf-8")))
                        if len(enc_json) == 0:
                            continue
                        decoded_enc = enc_json[0]
                    except JSONDecodeError:
                        print("[DEBUG] WARNING `enc_json` parsing faield!")
                    # pil_img = Image.open(io.BytesIO(enc.astype(bytes)))  # RGB
                    pil_img = Image.open(io.BytesIO(base64.b64decode(decoded_enc)))

                    image = np.array(pil_img)
                    if len(image.shape) == 2:  # gray image
                        raise ValueError(
                            f"The image shape with {image.shape} is "
                            f"not in acceptable"
                        )
                    images.append(image)
                text_generation_chat_input.prompt_images = images
                print(
                    "[DEBUG] input `prompt_images` type"
                    f"({type(text_generation_chat_input.prompt_images)}): "
                    f"{text_generation_chat_input.prompt_images}"
                )

            if input_name == "chat_history":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                chat_history_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `chat_history_str` type"
                    f"({type(chat_history_str)}): "
                    f"{chat_history_str}"
                )
                try:
                    text_generation_chat_input.chat_history = json.loads(
                        chat_history_str
                    )
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

            if input_name == "system_message":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_generation_chat_input.system_message = str(
                    input_tensor[0].decode("utf-8")
                )
                print(
                    "[DEBUG] input `system_message` type"
                    f"({type(text_generation_chat_input.system_message)}): "
                    f"{text_generation_chat_input.system_message}"
                )

            if input_name == "max_new_tokens":
                text_generation_chat_input.max_new_tokens = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `max_new_tokens` type"
                    f"({type(text_generation_chat_input.max_new_tokens)}): "
                    f"{text_generation_chat_input.max_new_tokens}"
                )

            if input_name == "top_k":
                text_generation_chat_input.top_k = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `top_k` type"
                    f"({type(text_generation_chat_input.top_k)}): "
                    f"{text_generation_chat_input.top_k}"
                )

            if input_name == "temperature":
                text_generation_chat_input.temperature = struct.unpack(
                    "f", b_input_tensor
                )[0]
                print(
                    "[DEBUG] input `temperature` type"
                    f"({type(text_generation_chat_input.temperature)}): "
                    f"{text_generation_chat_input.temperature}"
                )
                text_generation_chat_input.temperature = round(
                    text_generation_chat_input.temperature, 2
                )

            if input_name == "random_seed":
                text_generation_chat_input.random_seed = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `random_seed` type"
                    f"({type(text_generation_chat_input.random_seed)}): "
                    f"{text_generation_chat_input.random_seed}"
                )

            if input_name == "extra_params":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                extra_params_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `extra_params` type"
                    f"({type(extra_params_str)}): "
                    f"{extra_params_str}"
                )

                try:
                    text_generation_chat_input.extra_params = json.loads(
                        extra_params_str
                    )
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

        return text_generation_chat_input

    @staticmethod
    def parse_task_text_generation_chat_output(sequences: list):
        text_outputs = [seq["generated_text"].encode("utf-8") for seq in sequences]

        return serialize_byte_tensor(np.asarray(text_outputs))

    @staticmethod
    def parse_task_visual_question_answering_input(
        request,
    ) -> VisualQuestionAnsweringInput:
        text_visual_question_answering_input = VisualQuestionAnsweringInput()

        for i, b_input_tensor in zip(request.inputs, request.raw_input_contents):
            input_name = i.name

            if input_name == "prompt":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_visual_question_answering_input.prompt = str(
                    input_tensor[0].decode("utf-8")
                )
                print(
                    "[DEBUG] input `prompt` type"
                    f"({type(text_visual_question_answering_input.prompt)}): "
                    f"{text_visual_question_answering_input.prompt}"
                )

            if input_name == "prompt_images":
                input_tensors = deserialize_bytes_tensor(b_input_tensor)
                images = []
                for enc in input_tensors:
                    if len(enc) == 0:
                        continue
                    try:
                        enc_json = json.loads(str(enc.decode("utf-8")))
                        if len(enc_json) == 0:
                            continue
                        decoded_enc = enc_json[0]
                    except JSONDecodeError:
                        print("[DEBUG] WARNING `enc_json` parsing faield!")
                    # pil_img = Image.open(io.BytesIO(enc.astype(bytes)))  # RGB
                    pil_img = Image.open(io.BytesIO(base64.b64decode(decoded_enc)))

                    image = np.array(pil_img)
                    if len(image.shape) == 2:  # gray image
                        raise ValueError(
                            f"The image shape with {image.shape} is "
                            f"not in acceptable"
                        )
                    images.append(image)
                # TODO: check wethere there are issues in batch size dimention
                text_visual_question_answering_input.prompt_images = images
                print(
                    "[DEBUG] input `prompt_images` type"
                    f"({type(text_visual_question_answering_input.prompt_images)}): "
                    f"{text_visual_question_answering_input.prompt_images}"
                )

            if input_name == "chat_history":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                chat_history_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `chat_history_str` type"
                    f"({type(chat_history_str)}): "
                    f"{chat_history_str}"
                )
                try:
                    text_visual_question_answering_input.chat_history = json.loads(
                        chat_history_str
                    )
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

            if input_name == "system_message":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                text_visual_question_answering_input.system_message = str(
                    input_tensor[0].decode("utf-8")
                )
                print(
                    "[DEBUG] input `system_message` type"
                    f"({type(text_visual_question_answering_input.system_message)}): "
                    f"{text_visual_question_answering_input.system_message}"
                )

            if input_name == "max_new_tokens":
                text_visual_question_answering_input.max_new_tokens = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `max_new_tokens` type"
                    f"({type(text_visual_question_answering_input.max_new_tokens)}): "
                    f"{text_visual_question_answering_input.max_new_tokens}"
                )

            if input_name == "top_k":
                text_visual_question_answering_input.top_k = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `top_k` type"
                    f"({type(text_visual_question_answering_input.top_k)}): "
                    f"{text_visual_question_answering_input.top_k}"
                )

            if input_name == "temperature":
                text_visual_question_answering_input.temperature = struct.unpack(
                    "f", b_input_tensor
                )[0]
                print(
                    "[DEBUG] input `temperature` type"
                    f"({type(text_visual_question_answering_input.temperature)}): "
                    f"{text_visual_question_answering_input.temperature}"
                )
                text_visual_question_answering_input.temperature = round(
                    text_visual_question_answering_input.temperature, 2
                )

            if input_name == "random_seed":
                text_visual_question_answering_input.random_seed = int.from_bytes(
                    b_input_tensor, "little"
                )
                print(
                    "[DEBUG] input `random_seed` type"
                    f"({type(text_visual_question_answering_input.random_seed)}): "
                    f"{text_visual_question_answering_input.random_seed}"
                )

            if input_name == "extra_params":
                input_tensor = deserialize_bytes_tensor(b_input_tensor)
                extra_params_str = str(input_tensor[0].decode("utf-8"))
                print(
                    "[DEBUG] input `extra_params` type"
                    f"({type(extra_params_str)}): "
                    f"{extra_params_str}"
                )

                try:
                    text_visual_question_answering_input.extra_params = json.loads(
                        extra_params_str
                    )
                except JSONDecodeError:
                    print("[DEBUG] WARNING `extra_params` parsing faield!")
                    continue

        return text_visual_question_answering_input

    @staticmethod
    def parse_task_visual_question_answering_output(sequences: list):
        text_outputs = [seq["generated_text"].encode("utf-8") for seq in sequences]

        return serialize_byte_tensor(np.asarray(text_outputs))


class RawIO:
    @staticmethod
    def parse_byte_tensor(byte_tensor) -> List[str]:
        input_tensors = deserialize_bytes_tensor(byte_tensor)
        outs = [str(tensor.decode("utf-8")) for tensor in input_tensors]

        return outs

    @staticmethod
    def parse_unsigned_int_tensor(int_tensor) -> int:
        return int.from_bytes(int_tensor, "little")

    @staticmethod
    def parse_signed_int_tensor(int_tensor) -> int:
        return int.from_bytes(int_tensor, "little", signed=True)

    @staticmethod
    def parse_float_tensor(float_tensor) -> float:
        return struct.unpack("f", float_tensor)[0]

    @staticmethod
    def parse_boolean_tensor(bool_tensor) -> bool:
        return struct.unpack("?", bool_tensor)[0]



# pylint: disable=no-member,wrong-import-position,too-many-lines,no-name-in-module
from typing import Dict, Union

from google.protobuf import field_mask_pb2
from google.protobuf.struct_pb2 import Struct

# common
import instill.protogen.common.healthcheck.v1beta.healthcheck_pb2 as healthcheck
import instill.protogen.vdp.pipeline.v1beta.component_definition_pb2 as component_definition

# pipeline
import instill.protogen.vdp.pipeline.v1beta.pipeline_pb2 as pipeline_interface
import instill.protogen.vdp.pipeline.v1beta.pipeline_public_service_pb2_grpc as pipeline_service
import instill.protogen.vdp.pipeline.v1beta.secret_pb2 as secret_interface
from instill.clients.base import Client, RequestFactory
from instill.clients.constant import DEFAULT_INSTANCE
from instill.clients.instance import InstillInstance
from instill.configuration import global_config
from instill.protogen.vdp.pipeline.v1beta import common_pb2
from instill.utils.error_handler import grpc_handler

# from instill.utils.logger import Logger


class PipelineClient(Client):
    def __init__(
        self, namespace: str, async_enabled: bool, target_namespace: str = ""
    ) -> None:
        self.hosts: Dict[str, InstillInstance] = {}
        self.namespace: str = namespace
        self.target_namespace: str = (
            namespace if target_namespace == "" else target_namespace
        )
        if DEFAULT_INSTANCE in global_config.hosts:
            self.instance = DEFAULT_INSTANCE
        elif len(global_config.hosts) == 0:
            self.instance = ""
        else:
            self.instance = list(global_config.hosts.keys())[0]

        if global_config.hosts is not None:
            for instance, config in global_config.hosts.items():
                self.hosts[instance] = InstillInstance(
                    pipeline_service.PipelinePublicServiceStub,
                    url=config.url,
                    token=config.token,
                    secure=config.secure,
                    async_enabled=async_enabled,
                )

    @property
    def hosts(self):
        return self._hosts

    @hosts.setter
    def hosts(self, hosts: Dict[str, InstillInstance]):
        self._hosts = hosts

    @property
    def instance(self):
        return self._instance

    @instance.setter
    def instance(self, instance: str):
        self._instance = instance

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, metadata: str):
        self._metadata = metadata

    def liveness(
        self, async_enabled: bool = False
    ) -> pipeline_interface.LivenessResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.Liveness,
                request=pipeline_interface.LivenessRequest(),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.Liveness,
            request=pipeline_interface.LivenessRequest(),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    def readiness(
        self, async_enabled: bool = False
    ) -> pipeline_interface.ReadinessResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.Readiness,
                request=pipeline_interface.ReadinessRequest(),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.Readiness,
            request=pipeline_interface.ReadinessRequest(),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    def is_serving(self) -> bool:
        try:
            return (
                self.readiness().health_check_response.status
                == healthcheck.HealthCheckResponse.SERVING_STATUS_SERVING
            )
        except Exception:
            return False

    @grpc_handler
    def create_pipeline(
        self,
        name: str,
        description: str,
        recipe: dict,
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateUserPipelineResponse:
        pipeline = pipeline_interface.Pipeline(
            id=name,
            description=description,
        )
        pipeline.recipe.update(recipe)
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CreateUserPipeline,
                request=pipeline_interface.CreateUserPipelineRequest(
                    pipeline=pipeline, parent=self.target_namespace
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateUserPipeline,
            request=pipeline_interface.CreateUserPipelineRequest(
                pipeline=pipeline, parent=self.target_namespace
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetUserPipeline,
                request=pipeline_interface.GetUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetUserPipeline,
            request=pipeline_interface.GetUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def lookup_pipeline(
        self,
        pipeline_uid: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.LookUpPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.LookUpPipeline,
                request=pipeline_interface.LookUpPipelineRequest(
                    permalink=f"pipelines/{pipeline_uid}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.LookUpPipeline,
            request=pipeline_interface.LookUpPipelineRequest(
                permalink=f"pipelines/{pipeline_uid}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def rename_pipeline(
        self,
        name: str,
        new_name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RenameUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.RenameUserPipeline,
                request=pipeline_interface.RenameUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    new_pipeline_id=new_name,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RenameUserPipeline,
            request=pipeline_interface.RenameUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                new_pipeline_id=new_name,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_pipeline(
        self,
        pipeline: pipeline_interface.Pipeline,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.UpdateUserPipeline,
                request=pipeline_interface.UpdateUserPipelineRequest(
                    pipeline=pipeline,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateUserPipeline,
            request=pipeline_interface.UpdateUserPipelineRequest(
                pipeline=pipeline,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def validate_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.ValidateUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ValidateUserPipeline,
                request=pipeline_interface.ValidateUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ValidateUserPipeline,
            request=pipeline_interface.ValidateUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def clone_pipeline(
        self,
        name: str,
        target: str,
        description: str,
        sharing: common_pb2.Sharing,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CloneUserPipeline,
                request=pipeline_interface.CloneUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    target=f"{self.target_namespace}/pipelines/{target}",
                    description=description,
                    sharing=sharing,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CloneUserPipeline,
            request=pipeline_interface.CloneUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                target=f"{self.target_namespace}/pipelines/{target}",
                description=description,
                sharing=sharing,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_pipeline(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerUserPipelineResponse:
        request = pipeline_interface.TriggerUserPipelineRequest(
            name=f"{self.target_namespace}/pipelines/{name}",
        )
        for input_value in inputs:
            trigger_inputs = Struct()
            trigger_inputs.update(input_value)
            request.inputs.append(trigger_inputs)
        for d in data:
            trigger_data = pipeline_interface.TriggerData()
            trigger_data.variable.update(d)
            request.data.append(trigger_data)

        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.TriggerUserPipeline,
                request=request,
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerUserPipeline,
            request=request,
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_async_pipeline(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.TriggerAsyncUserPipeline,
                request=pipeline_interface.TriggerAsyncUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerAsyncUserPipeline,
            request=pipeline_interface.TriggerAsyncUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteUserPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.DeleteUserPipeline,
                request=pipeline_interface.DeleteUserPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteUserPipeline,
            request=pipeline_interface.DeleteUserPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_pipelines(
        self,
        visibility: pipeline_interface.Pipeline.Visibility.ValueType,
        order_by: str,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        public=False,
        async_enabled: bool = False,
    ) -> Union[
        pipeline_interface.ListPipelinesResponse,
        pipeline_interface.ListUserPipelinesResponse,
    ]:
        if async_enabled:
            if public:
                method = self.hosts[self.instance].async_client.ListPipelines
                return RequestFactory(
                    method=method,
                    request=pipeline_interface.ListPipelinesRequest(
                        filter=filter_str,
                        page_size=total_size,
                        page_token=next_page_token,
                        show_deleted=show_deleted,
                        view=pipeline_interface.Pipeline.VIEW_RECIPE,
                        visibility=visibility,
                        order_by=order_by,
                    ),
                    metadata=self.hosts[self.instance].metadata,
                ).send_async()
            method = self.hosts[self.instance].async_client.ListUserPipelines
            return RequestFactory(
                method=method,
                request=pipeline_interface.ListUserPipelinesRequest(
                    parent=self.target_namespace,
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    visibility=visibility,
                    order_by=order_by,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()
        if public:
            method = self.hosts[self.instance].client.ListPipelines
            return RequestFactory(
                method=method,
                request=pipeline_interface.ListPipelinesRequest(
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                    visibility=visibility,
                    order_by=order_by,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_sync()
        method = self.hosts[self.instance].client.ListUserPipelines
        return RequestFactory(
            method=method,
            request=pipeline_interface.ListUserPipelinesRequest(
                parent=self.target_namespace,
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
                visibility=visibility,
                order_by=order_by,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_operation(
        self,
        operation_id: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetOperationResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetOperation,
                request=pipeline_interface.GetOperationRequest(
                    operation_id=operation_id,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetOperation,
            request=pipeline_interface.GetOperationRequest(
                operation_id=operation_id,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def create_pipeline_release(
        self,
        version: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateUserPipelineReleaseResponse:
        """Create a release version of a pipeline

        Args:
            name (str): Must be a sematic version vX.Y.Z

        Returns:
            pipeline_interface.PipelineRelease: Released pipeline
        """
        pipeline_release = pipeline_interface.PipelineRelease(
            id=version,
        )
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CreateUserPipelineRelease,
                request=pipeline_interface.CreateUserPipelineReleaseRequest(
                    release=pipeline_release, parent=self.target_namespace
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateUserPipelineRelease,
            request=pipeline_interface.CreateUserPipelineReleaseRequest(
                release=pipeline_release, parent=self.target_namespace
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetUserPipelineReleaseResponse:
        """Get a released pipeline

        Args:
            name (str): Must have the format of {name}/releases/*

        Returns:
            pipeline_interface.Pipeline: Released pipeline
        """
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetUserPipelineRelease,
                request=pipeline_interface.GetUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetUserPipelineRelease,
            request=pipeline_interface.GetUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def rename_pipeline_release(
        self,
        name: str,
        new_version: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RenameUserPipelineReleaseResponse:
        """Rename a released pipeline

        Args:
            name (str): Must have the format of {name}/releases/*
            new_version (str): New release version, must be a sematic version vX.Y.Z

        Returns:
            pipeline_interface.PipelineRelease: released pipeline
        """
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.RenameUserPipelineRelease,
                request=pipeline_interface.RenameUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    new_pipeline_release_id=new_version,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RenameUserPipelineRelease,
            request=pipeline_interface.RenameUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                new_pipeline_release_id=new_version,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_pipeline_release(
        self,
        pipeline_release: pipeline_interface.PipelineRelease,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.UpdateUserPipelineRelease,
                request=pipeline_interface.UpdateUserPipelineReleaseRequest(
                    release=pipeline_release,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateUserPipelineRelease,
            request=pipeline_interface.UpdateUserPipelineReleaseRequest(
                release=pipeline_release,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_pipeline_releases(
        self,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListUserPipelineReleasesResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListUserPipelineReleases,
                request=pipeline_interface.ListUserPipelineReleasesRequest(
                    parent=self.target_namespace,
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ListUserPipelineReleases,
            request=pipeline_interface.ListUserPipelineReleasesRequest(
                parent=self.target_namespace,
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.DeleteUserPipelineRelease,
                request=pipeline_interface.DeleteUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteUserPipelineRelease,
            request=pipeline_interface.DeleteUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def restore_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RestoreUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.RestoreUserPipelineRelease,
                request=pipeline_interface.RestoreUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RestoreUserPipelineRelease,
            request=pipeline_interface.RestoreUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_pipeline_release(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerUserPipelineRelease,
                request=pipeline_interface.TriggerUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerUserPipelineRelease,
            request=pipeline_interface.TriggerUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_async_pipeline_release(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerAsyncUserPipelineRelease,
                request=pipeline_interface.TriggerAsyncUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerAsyncUserPipelineRelease,
            request=pipeline_interface.TriggerAsyncUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def create_secret(
        self,
        name: str,
        value: str,
        async_enabled: bool = False,
    ) -> secret_interface.CreateUserSecretResponse:
        secret = secret_interface.Secret(id=name, value=value)
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CreateUserSecret,
                request=secret_interface.CreateUserSecretRequest(
                    secret=secret,
                    parent=self.target_namespace,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateUserSecret,
            request=secret_interface.CreateUserSecretRequest(
                secret=secret,
                parent=self.target_namespace,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_secret(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> secret_interface.GetUserSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetUserSecret,
                request=secret_interface.GetUserSecretRequest(
                    name=f"{self.target_namespace}/secrets/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetUserSecret,
            request=secret_interface.GetUserSecretRequest(
                name=f"{self.target_namespace}/secrets/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_secrets(
        self,
        secret: secret_interface.Secret,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> secret_interface.UpdateUserSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.UpdateUserSecret,
                request=secret_interface.UpdateUserSecretRequest(
                    secret=secret,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateUserSecret,
            request=secret_interface.UpdateUserSecretRequest(
                secret=secret,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_secret(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> secret_interface.DeleteUserSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.DeleteUserSecret,
                request=secret_interface.DeleteUserSecretRequest(
                    name=f"{self.target_namespace}/secrets/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteUserSecret,
            request=secret_interface.DeleteUserSecretRequest(
                name=f"{self.target_namespace}/secrets/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_secrets(
        self,
        next_page_token: str = "",
        total_size: int = 100,
        async_enabled: bool = False,
    ) -> secret_interface.ListUserSecretsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListUserSecrets,
                request=secret_interface.ListUserSecretsRequest(
                    parent=self.target_namespace,
                    page_size=total_size,
                    page_token=next_page_token,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()
        return RequestFactory(
            method=self.hosts[self.instance].client.ListUserSecrets,
            request=secret_interface.ListUserSecretsRequest(
                parent=self.target_namespace,
                page_size=total_size,
                page_token=next_page_token,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    ######## organization endpoints

    @grpc_handler
    def create_org_pipeline(
        self,
        name: str,
        recipe: Struct,
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateOrganizationPipelineResponse:
        pipeline = pipeline_interface.Pipeline(
            id=name,
            recipe=recipe,
        )
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.CreateOrganizationPipeline,
                request=pipeline_interface.CreateOrganizationPipelineRequest(
                    pipeline=pipeline, parent=self.target_namespace
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateOrganizationPipeline,
            request=pipeline_interface.CreateOrganizationPipelineRequest(
                pipeline=pipeline, parent=self.target_namespace
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_org_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetOrganizationPipeline,
                request=pipeline_interface.GetOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetOrganizationPipeline,
            request=pipeline_interface.GetOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def rename_org_pipeline(
        self,
        name: str,
        new_name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RenameOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.RenameOrganizationPipeline,
                request=pipeline_interface.RenameOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    new_pipeline_id=new_name,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RenameOrganizationPipeline,
            request=pipeline_interface.RenameOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                new_pipeline_id=new_name,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_org_pipeline(
        self,
        pipeline: pipeline_interface.Pipeline,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.UpdateOrganizationPipeline,
                request=pipeline_interface.UpdateOrganizationPipelineRequest(
                    pipeline=pipeline,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateOrganizationPipeline,
            request=pipeline_interface.UpdateOrganizationPipelineRequest(
                pipeline=pipeline,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def validate_org_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.ValidateOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.ValidateOrganizationPipeline,
                request=pipeline_interface.ValidateOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ValidateOrganizationPipeline,
            request=pipeline_interface.ValidateOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def clone_org_pipeline(
        self,
        name: str,
        target: str,
        description: str,
        sharing: common_pb2.Sharing,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CloneOrganizationPipeline,
                request=pipeline_interface.CloneOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    target=f"{self.target_namespace}/pipelines/{target}",
                    description=description,
                    sharing=sharing,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CloneOrganizationPipeline,
            request=pipeline_interface.CloneOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                target=f"{self.target_namespace}/pipelines/{target}",
                description=description,
                sharing=sharing,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_org_pipeline(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerOrganizationPipeline,
                request=pipeline_interface.TriggerOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerOrganizationPipeline,
            request=pipeline_interface.TriggerOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_org_async_pipeline(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerAsyncOrganizationPipeline,
                request=pipeline_interface.TriggerAsyncOrganizationPipelineRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerAsyncOrganizationPipeline,
            request=pipeline_interface.TriggerAsyncOrganizationPipelineRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_org_pipeline(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteOrganizationPipelineResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.DeleteOrganizationPipeline,
                request=pipeline_interface.DeleteOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteOrganizationPipeline,
            request=pipeline_interface.DeleteOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_org_pipelines(
        self,
        visibility: pipeline_interface.Pipeline.Visibility.ValueType,
        order_by: str,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListOrganizationPipelinesResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListOrganizationPipelines,
                request=pipeline_interface.ListOrganizationPipelinesRequest(
                    visibility=visibility,
                    order_by=order_by,
                    parent=self.target_namespace,
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()
        return RequestFactory(
            method=self.hosts[self.instance].client.ListOrganizationPipelines,
            request=pipeline_interface.ListOrganizationPipelinesRequest(
                visibility=visibility,
                order_by=order_by,
                parent=self.target_namespace,
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def create_org_pipeline_release(
        self,
        version: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.CreateOrganizationPipelineReleaseResponse:
        """Create a release version of a pipeline

        Args:
            name (str): Must be a sematic version vX.Y.Z

        Returns:
            pipeline_interface.PipelineRelease: Released pipeline
        """
        pipeline_release = pipeline_interface.PipelineRelease(
            id=version,
        )
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.CreateOrganizationPipelineRelease,
                request=pipeline_interface.CreateOrganizationPipelineReleaseRequest(
                    release=pipeline_release, parent=self.target_namespace
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateOrganizationPipelineRelease,
            request=pipeline_interface.CreateOrganizationPipelineReleaseRequest(
                release=pipeline_release, parent=self.target_namespace
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_org_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetOrganizationPipelineReleaseResponse:
        """Get a released pipeline

        Args:
            name (str): Must have the format of {name}/releases/*

        Returns:
            pipeline_interface.Pipeline: Released pipeline
        """
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.GetOrganizationPipelineRelease,
                request=pipeline_interface.GetOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetOrganizationPipelineRelease,
            request=pipeline_interface.GetOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def rename_org_pipeline_release(
        self,
        name: str,
        new_version: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RenameOrganizationPipelineReleaseResponse:
        """Rename a released pipeline

        Args:
            name (str): Must have the format of {name}/releases/*
            new_version (str): New release version, must be a sematic version vX.Y.Z

        Returns:
            pipeline_interface.PipelineRelease: released pipeline
        """
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.RenameOrganizationPipelineRelease,
                request=pipeline_interface.RenameOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    new_pipeline_release_id=new_version,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RenameOrganizationPipelineRelease,
            request=pipeline_interface.RenameOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                new_pipeline_release_id=new_version,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_org_pipeline_release(
        self,
        pipeline_release: pipeline_interface.PipelineRelease,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> pipeline_interface.UpdateOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.UpdateOrganizationPipelineRelease,
                request=pipeline_interface.UpdateOrganizationPipelineReleaseRequest(
                    release=pipeline_release,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateOrganizationPipelineRelease,
            request=pipeline_interface.UpdateOrganizationPipelineReleaseRequest(
                release=pipeline_release,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_org_pipeline_releases(
        self,
        filter_str: str = "",
        next_page_token: str = "",
        total_size: int = 100,
        show_deleted: bool = False,
        async_enabled: bool = False,
    ) -> pipeline_interface.ListOrganizationPipelineReleasesResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.ListOrganizationPipelineReleases,
                request=pipeline_interface.ListOrganizationPipelineReleasesRequest(
                    parent=self.target_namespace,
                    filter=filter_str,
                    page_size=total_size,
                    page_token=next_page_token,
                    show_deleted=show_deleted,
                    view=pipeline_interface.Pipeline.VIEW_RECIPE,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ListOrganizationPipelineReleases,
            request=pipeline_interface.ListOrganizationPipelineReleasesRequest(
                parent=self.target_namespace,
                filter=filter_str,
                page_size=total_size,
                page_token=next_page_token,
                show_deleted=show_deleted,
                view=pipeline_interface.Pipeline.VIEW_RECIPE,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_org_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.DeleteOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.DeleteOrganizationPipelineRelease,
                request=pipeline_interface.DeleteOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}"
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteOrganizationPipelineRelease,
            request=pipeline_interface.DeleteOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}"
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def restore_org_pipeline_release(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> pipeline_interface.RestoreOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.RestoreOrganizationPipelineRelease,
                request=pipeline_interface.RestoreOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.RestoreOrganizationPipelineRelease,
            request=pipeline_interface.RestoreOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_org_pipeline_release(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerOrganizationPipelineRelease,
                request=pipeline_interface.TriggerOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerOrganizationPipelineRelease,
            request=pipeline_interface.TriggerOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_async_org_pipeline_release(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerAsyncOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerAsyncOrganizationPipelineRelease,
                request=pipeline_interface.TriggerOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[
                self.instance
            ].client.TriggerAsyncOrganizationPipelineRelease,
            request=pipeline_interface.TriggerOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def create_org_secret(
        self,
        name: str,
        value: str,
        async_enabled: bool = False,
    ) -> secret_interface.CreateOrganizationSecretResponse:
        secret = secret_interface.Secret(id=name, value=value)
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CreateOrganizationSecret,
                request=secret_interface.CreateUserSecretRequest(
                    secret=secret,
                    parent=self.target_namespace,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CreateOrganizationSecret,
            request=secret_interface.CreateUserSecretRequest(
                secret=secret,
                parent=self.target_namespace,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_org_secret(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> secret_interface.GetOrganizationSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetOrganizationSecret,
                request=secret_interface.GetOrganizationSecretRequest(
                    name=f"{self.target_namespace}/secrets/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetOrganizationSecret,
            request=secret_interface.GetOrganizationSecretRequest(
                name=f"{self.target_namespace}/secrets/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def update_org_secrets(
        self,
        secret: secret_interface.Secret,
        mask: field_mask_pb2.FieldMask,
        async_enabled: bool = False,
    ) -> secret_interface.UpdateOrganizationSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.UpdateOrganizationSecret,
                request=secret_interface.UpdateOrganizationSecretRequest(
                    secret=secret,
                    update_mask=mask,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.UpdateOrganizationSecret,
            request=secret_interface.UpdateOrganizationSecretRequest(
                secret=secret,
                update_mask=mask,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def delete_org_secret(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> secret_interface.DeleteOrganizationSecretResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.DeleteOrganizationSecret,
                request=secret_interface.DeleteOrganizationSecretRequest(
                    name=f"{self.target_namespace}/secrets/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.DeleteOrganizationSecret,
            request=secret_interface.DeleteOrganizationSecretRequest(
                name=f"{self.target_namespace}/secrets/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_org_secrets(
        self,
        next_page_token: str = "",
        total_size: int = 100,
        async_enabled: bool = False,
    ) -> secret_interface.ListOrganizationSecretsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListOrganizationSecrets,
                request=secret_interface.ListOrganizationSecretsRequest(
                    parent=self.target_namespace,
                    page_size=total_size,
                    page_token=next_page_token,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()
        return RequestFactory(
            method=self.hosts[self.instance].client.ListOrganizationSecrets,
            request=secret_interface.ListOrganizationSecretsRequest(
                parent=self.target_namespace,
                page_size=total_size,
                page_token=next_page_token,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_hub_stats(
        self,
        async_enabled: bool = False,
    ) -> pipeline_interface.GetHubStatsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetHubStats,
                request=pipeline_interface.GetHubStatsRequest(),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetHubStats,
            request=pipeline_interface.GetHubStatsRequest(),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def clone_pipeline_release(
        self,
        name: str,
        target: str,
        description: str,
        sharing: common_pb2.Sharing,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneUserPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CloneUserPipelineRelease,
                request=pipeline_interface.CloneUserPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    target=f"{self.target_namespace}/pipelines/{target}",
                    description=description,
                    sharing=sharing,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CloneUserPipelineRelease,
            request=pipeline_interface.CloneUserPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                target=f"{self.target_namespace}/pipelines/{target}",
                description=description,
                sharing=sharing,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_pipeline_with_stream(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerUserPipelineWithStreamResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerUserPipelineWithStream,
                request=pipeline_interface.TriggerUserPipelineWithStreamRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerUserPipelineWithStream,
            request=pipeline_interface.TriggerUserPipelineWithStreamRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def clone_organization_pipeline_release(
        self,
        name: str,
        target: str,
        description: str,
        sharing: common_pb2.Sharing,
        async_enabled: bool = False,
    ) -> pipeline_interface.CloneOrganizationPipelineReleaseResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.CloneOrganizationPipelineRelease,
                request=pipeline_interface.CloneOrganizationPipelineReleaseRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    target=f"{self.target_namespace}/pipelines/{target}",
                    description=description,
                    sharing=sharing,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CloneOrganizationPipelineRelease,
            request=pipeline_interface.CloneOrganizationPipelineReleaseRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                target=f"{self.target_namespace}/pipelines/{target}",
                description=description,
                sharing=sharing,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def trigger_organization_pipeline_stream(
        self,
        name: str,
        inputs: list,
        data: list,
        async_enabled: bool = False,
    ) -> pipeline_interface.TriggerOrganizationPipelineStreamResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[
                    self.instance
                ].async_client.TriggerOrganizationPipelineStream,
                request=pipeline_interface.TriggerOrganizationPipelineStreamRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    inputs=inputs,
                    data=data,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.TriggerOrganizationPipelineStream,
            request=pipeline_interface.TriggerOrganizationPipelineStreamRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                inputs=inputs,
                data=data,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_connector_definitions(
        self,
        total_size: int = 100,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListConnectorDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListConnectorDefinitions,
                request=component_definition.ListConnectorDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ListConnectorDefinitions,
            request=component_definition.ListConnectorDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_connector_definition(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> component_definition.GetConnectorDefinitionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetConnectorDefinition,
                request=component_definition.GetConnectorDefinitionRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetConnectorDefinition,
            request=component_definition.GetConnectorDefinitionRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=component_definition.ComponentDefinition.VIEW_FULL,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_operator_definitions(
        self,
        total_size: int = 100,
        next_page_token: str = "",
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListOperatorDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListOperatorDefinitions,
                request=component_definition.ListOperatorDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page_size=total_size,
                    page_token=next_page_token,
                    filter=filter_str,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ListOperatorDefinitions,
            request=component_definition.ListOperatorDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page_size=total_size,
                page_token=next_page_token,
                filter=filter_str,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def list_component_definitions(
        self,
        page: int,
        total_size: int = 100,
        filter_str: str = "",
        async_enabled: bool = False,
    ) -> component_definition.ListComponentDefinitionsResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.ListComponentDefinitions,
                request=component_definition.ListComponentDefinitionsRequest(
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                    page=page,
                    page_size=total_size,
                    filter=filter_str,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.ListComponentDefinitions,
            request=component_definition.ListComponentDefinitionsRequest(
                view=component_definition.ComponentDefinition.VIEW_FULL,
                page=page,
                page_size=total_size,
                filter=filter_str,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def get_operator_definition(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> component_definition.GetOperatorDefinitionResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.GetOperatorDefinition,
                request=component_definition.GetOperatorDefinitionRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                    view=component_definition.ComponentDefinition.VIEW_FULL,
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.GetOperatorDefinition,
            request=component_definition.GetOperatorDefinitionRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
                view=component_definition.ComponentDefinition.VIEW_FULL,
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()

    @grpc_handler
    def check_name(
        self,
        name: str,
        async_enabled: bool = False,
    ) -> common_pb2.CheckNameResponse:
        if async_enabled:
            return RequestFactory(
                method=self.hosts[self.instance].async_client.CheckName,
                request=common_pb2.CheckNameRequest(
                    name=f"{self.target_namespace}/pipelines/{name}",
                ),
                metadata=self.hosts[self.instance].metadata,
            ).send_async()

        return RequestFactory(
            method=self.hosts[self.instance].client.CheckName,
            request=common_pb2.CheckNameRequest(
                name=f"{self.target_namespace}/pipelines/{name}",
            ),
            metadata=self.hosts[self.instance].metadata,
        ).send_sync()



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,no-name-in-module

import instill.protogen.core.mgmt.v1beta.mgmt_public_service_pb2_grpc as mgmt_service
import instill.protogen.model.model.v1alpha.model_public_service_pb2_grpc as model_service
import instill.protogen.vdp.pipeline.v1beta.pipeline_public_service_pb2_grpc as pipeline_service
from instill.clients import MgmtClient, ModelClient, PipelineClient
from instill.clients.instance import InstillInstance


def describe_client():
    def describe_instance():
        def when_not_set(expect):
            mgmt_client = MgmtClient(False)
            expect(mgmt_client.instance) == ""
            model_client = ModelClient(namespace="", async_enabled=False)
            expect(model_client.instance) == ""
            pipeline_client = PipelineClient(namespace="", async_enabled=False)
            expect(pipeline_client.instance) == ""

        def when_set_correct_type(expect):
            mgmt_client = MgmtClient(False)
            mgmt_client.instance = "staging"
            expect(mgmt_client.instance) == "staging"
            model_client = ModelClient(namespace="", async_enabled=False)
            model_client.instance = "staging"
            expect(model_client.instance) == "staging"
            pipeline_client = PipelineClient(namespace="", async_enabled=False)
            pipeline_client.instance = "staging"
            expect(pipeline_client.instance) == "staging"

    def describe_host():
        def when_not_set(expect):
            mgmt_client = MgmtClient(False)
            expect(mgmt_client.hosts) is None
            model_client = ModelClient(namespace="", async_enabled=False)
            expect(model_client.hosts) is None
            pipeline_client = PipelineClient(namespace="", async_enabled=False)
            expect(pipeline_client.hosts) is None

        def when_set_correct_type_url(expect):
            mgmt_instance = {
                "test_instance": InstillInstance(
                    mgmt_service.MgmtPublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            mgmt_client = MgmtClient(False)
            mgmt_client.hosts = mgmt_instance
            expect(mgmt_client.hosts["test_instance"].url) == "test_url"

            model_instance = {
                "test_instance": InstillInstance(
                    model_service.ModelPublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            model_client = ModelClient(namespace="", async_enabled=False)
            model_client.hosts = model_instance
            expect(model_client.hosts["test_instance"].url) == "test_url"

            pipeline_instance = {
                "test_instance": InstillInstance(
                    pipeline_service.PipelinePublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            pipeline_client = PipelineClient(namespace="", async_enabled=False)
            pipeline_client.hosts = pipeline_instance
            expect(pipeline_client.hosts["test_instance"].url) == "test_url"

        def when_set_correct_type_token(expect):
            mgmt_instance = {
                "test_instance": InstillInstance(
                    mgmt_service.MgmtPublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            mgmt_client = MgmtClient(False)
            mgmt_client.hosts = mgmt_instance
            expect(mgmt_client.hosts["test_instance"].token) == "token"

            model_instance = {
                "test_instance": InstillInstance(
                    model_service.ModelPublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            model_client = ModelClient(namespace="", async_enabled=False)
            model_client.hosts = model_instance
            expect(model_client.hosts["test_instance"].token) == "token"

            pipeline_instance = {
                "test_instance": InstillInstance(
                    pipeline_service.PipelinePublicServiceStub,
                    "test_url",
                    "token",
                    False,
                    False,
                )
            }
            pipeline_client = PipelineClient(namespace="", async_enabled=False)
            pipeline_client.hosts = pipeline_instance
            expect(pipeline_client.hosts["test_instance"].token) == "token"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: model/model/v1alpha/model_private_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from model.model.v1alpha import model_pb2 as model_dot_model_dot_v1alpha_dot_model__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/model/model/v1alpha/model_private_service.proto\x12\x13model.model.v1alpha\x1a\x1fmodel/model/v1alpha/model.proto2\xd7\x08\n\x13ModelPrivateService\x12n\n\x0fListModelsAdmin\x12+.model.model.v1alpha.ListModelsAdminRequest\x1a,.model.model.v1alpha.ListModelsAdminResponse\"\x00\x12q\n\x10LookUpModelAdmin\x12,.model.model.v1alpha.LookUpModelAdminRequest\x1a-.model.model.v1alpha.LookUpModelAdminResponse\"\x00\x12\x8c\x01\n\x19\x44\x65ployNamespaceModelAdmin\x12\x35.model.model.v1alpha.DeployNamespaceModelAdminRequest\x1a\x36.model.model.v1alpha.DeployNamespaceModelAdminResponse\"\x00\x12}\n\x14\x44\x65ployUserModelAdmin\x12\x30.model.model.v1alpha.DeployUserModelAdminRequest\x1a\x31.model.model.v1alpha.DeployUserModelAdminResponse\"\x00\x12\x95\x01\n\x1c\x44\x65ployOrganizationModelAdmin\x12\x38.model.model.v1alpha.DeployOrganizationModelAdminRequest\x1a\x39.model.model.v1alpha.DeployOrganizationModelAdminResponse\"\x00\x12\x83\x01\n\x16UndeployUserModelAdmin\x12\x32.model.model.v1alpha.UndeployUserModelAdminRequest\x1a\x33.model.model.v1alpha.UndeployUserModelAdminResponse\"\x00\x12\x92\x01\n\x1bUndeployNamespaceModelAdmin\x12\x37.model.model.v1alpha.UndeployNamespaceModelAdminRequest\x1a\x38.model.model.v1alpha.UndeployNamespaceModelAdminResponse\"\x00\x12\x9b\x01\n\x1eUndeployOrganizationModelAdmin\x12:.model.model.v1alpha.UndeployOrganizationModelAdminRequest\x1a;.model.model.v1alpha.UndeployOrganizationModelAdminResponse\"\x00\x42\xe5\x01\n\x17\x63om.model.model.v1alphaB\x18ModelPrivateServiceProtoP\x01ZBgithub.com/instill-ai/protogen-go/model/model/v1alpha;modelv1alpha\xa2\x02\x03MMX\xaa\x02\x13Model.Model.V1alpha\xca\x02\x13Model\\Model\\V1alpha\xe2\x02\x1fModel\\Model\\V1alpha\\GPBMetadata\xea\x02\x15Model::Model::V1alphab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'model.model.v1alpha.model_private_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.model.model.v1alphaB\030ModelPrivateServiceProtoP\001ZBgithub.com/instill-ai/protogen-go/model/model/v1alpha;modelv1alpha\242\002\003MMX\252\002\023Model.Model.V1alpha\312\002\023Model\\Model\\V1alpha\342\002\037Model\\Model\\V1alpha\\GPBMetadata\352\002\025Model::Model::V1alpha'
  _globals['_MODELPRIVATESERVICE']._serialized_start=106
  _globals['_MODELPRIVATESERVICE']._serialized_end=1217
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

This is the POC for metric-backend service that supports
1. receiving usage reports from pipeline-backend and model-backend clients
2. uploading usage to InfluxDB
3. uploading usage to Stripe
4. responding usage/price enquiries from clients
"""
import google.protobuf.descriptor

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import collections.abc
import grpc
import grpc.aio
import model.model.v1alpha.model_pb2
import typing

_T = typing.TypeVar('_T')

class _MaybeAsyncIterator(collections.abc.AsyncIterator[_T], collections.abc.Iterator[_T], metaclass=abc.ABCMeta):
    ...

class _ServicerContext(grpc.ServicerContext, grpc.aio.ServicerContext):  # type: ignore
    ...

class ModelPrivateServiceStub:
    """Model service responds to internal access
    ========== Private API ==========
    """

    def __init__(self, channel: typing.Union[grpc.Channel, grpc.aio.Channel]) -> None: ...
    ListModelsAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.ListModelsAdminRequest,
        model.model.v1alpha.model_pb2.ListModelsAdminResponse,
    ]
    """ListModelsAdmin method receives a ListModelsAdminRequest message and
    returns a ListModelsAdminResponse
    """
    LookUpModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.LookUpModelAdminRequest,
        model.model.v1alpha.model_pb2.LookUpModelAdminResponse,
    ]
    """LookUpModelAdmin method receives a LookUpModelAdminRequest message and
    returns a LookUpModelAdminResponse
    """
    DeployNamespaceModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployNamespaceModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployNamespaceModelAdminResponse,
    ]
    """DeployNamespaceModelAdmin deploy a model to online state"""
    DeployUserModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployUserModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployUserModelAdminResponse,
    ]
    """DeployUserModelAdmin deploy a model to online state"""
    DeployOrganizationModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployOrganizationModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployOrganizationModelAdminResponse,
    ]
    """DeployOrganizationModelAdmin deploy a model to online state"""
    UndeployUserModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployUserModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployUserModelAdminResponse,
    ]
    """UndeployUserModelAdmin undeploy a model to offline state"""
    UndeployNamespaceModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminResponse,
    ]
    """UndeployNamespaceModelAdmin undeploy a model to offline state"""
    UndeployOrganizationModelAdmin: grpc.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminResponse,
    ]
    """UndeployOrganizationModelAdmin undeploy a model to offline state"""

class ModelPrivateServiceAsyncStub:
    """Model service responds to internal access
    ========== Private API ==========
    """

    ListModelsAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.ListModelsAdminRequest,
        model.model.v1alpha.model_pb2.ListModelsAdminResponse,
    ]
    """ListModelsAdmin method receives a ListModelsAdminRequest message and
    returns a ListModelsAdminResponse
    """
    LookUpModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.LookUpModelAdminRequest,
        model.model.v1alpha.model_pb2.LookUpModelAdminResponse,
    ]
    """LookUpModelAdmin method receives a LookUpModelAdminRequest message and
    returns a LookUpModelAdminResponse
    """
    DeployNamespaceModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployNamespaceModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployNamespaceModelAdminResponse,
    ]
    """DeployNamespaceModelAdmin deploy a model to online state"""
    DeployUserModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployUserModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployUserModelAdminResponse,
    ]
    """DeployUserModelAdmin deploy a model to online state"""
    DeployOrganizationModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.DeployOrganizationModelAdminRequest,
        model.model.v1alpha.model_pb2.DeployOrganizationModelAdminResponse,
    ]
    """DeployOrganizationModelAdmin deploy a model to online state"""
    UndeployUserModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployUserModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployUserModelAdminResponse,
    ]
    """UndeployUserModelAdmin undeploy a model to offline state"""
    UndeployNamespaceModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminResponse,
    ]
    """UndeployNamespaceModelAdmin undeploy a model to offline state"""
    UndeployOrganizationModelAdmin: grpc.aio.UnaryUnaryMultiCallable[
        model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminRequest,
        model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminResponse,
    ]
    """UndeployOrganizationModelAdmin undeploy a model to offline state"""

class ModelPrivateServiceServicer(metaclass=abc.ABCMeta):
    """Model service responds to internal access
    ========== Private API ==========
    """

    @abc.abstractmethod
    def ListModelsAdmin(
        self,
        request: model.model.v1alpha.model_pb2.ListModelsAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.ListModelsAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.ListModelsAdminResponse]]:
        """ListModelsAdmin method receives a ListModelsAdminRequest message and
        returns a ListModelsAdminResponse
        """
    @abc.abstractmethod
    def LookUpModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.LookUpModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.LookUpModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.LookUpModelAdminResponse]]:
        """LookUpModelAdmin method receives a LookUpModelAdminRequest message and
        returns a LookUpModelAdminResponse
        """
    @abc.abstractmethod
    def DeployNamespaceModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.DeployNamespaceModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.DeployNamespaceModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.DeployNamespaceModelAdminResponse]]:
        """DeployNamespaceModelAdmin deploy a model to online state"""
    @abc.abstractmethod
    def DeployUserModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.DeployUserModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.DeployUserModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.DeployUserModelAdminResponse]]:
        """DeployUserModelAdmin deploy a model to online state"""
    @abc.abstractmethod
    def DeployOrganizationModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.DeployOrganizationModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.DeployOrganizationModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.DeployOrganizationModelAdminResponse]]:
        """DeployOrganizationModelAdmin deploy a model to online state"""
    @abc.abstractmethod
    def UndeployUserModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.UndeployUserModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.UndeployUserModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.UndeployUserModelAdminResponse]]:
        """UndeployUserModelAdmin undeploy a model to offline state"""
    @abc.abstractmethod
    def UndeployNamespaceModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.UndeployNamespaceModelAdminResponse]]:
        """UndeployNamespaceModelAdmin undeploy a model to offline state"""
    @abc.abstractmethod
    def UndeployOrganizationModelAdmin(
        self,
        request: model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminRequest,
        context: _ServicerContext,
    ) -> typing.Union[model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminResponse, collections.abc.Awaitable[model.model.v1alpha.model_pb2.UndeployOrganizationModelAdminResponse]]:
        """UndeployOrganizationModelAdmin undeploy a model to offline state"""

def add_ModelPrivateServiceServicer_to_server(servicer: ModelPrivateServiceServicer, server: typing.Union[grpc.Server, grpc.aio.Server]) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: core/mgmt/v1beta/mgmt_private_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from core.mgmt.v1beta import mgmt_pb2 as core_dot_mgmt_dot_v1beta_dot_mgmt__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+core/mgmt/v1beta/mgmt_private_service.proto\x12\x10\x63ore.mgmt.v1beta\x1a\x1b\x63ore/mgmt/v1beta/mgmt.proto2\xda\x0b\n\x12MgmtPrivateService\x12\x65\n\x0eListUsersAdmin\x12\'.core.mgmt.v1beta.ListUsersAdminRequest\x1a(.core.mgmt.v1beta.ListUsersAdminResponse\"\x00\x12_\n\x0cGetUserAdmin\x12%.core.mgmt.v1beta.GetUserAdminRequest\x1a&.core.mgmt.v1beta.GetUserAdminResponse\"\x00\x12h\n\x0fLookUpUserAdmin\x12(.core.mgmt.v1beta.LookUpUserAdminRequest\x1a).core.mgmt.v1beta.LookUpUserAdminResponse\"\x00\x12}\n\x16ListOrganizationsAdmin\x12/.core.mgmt.v1beta.ListOrganizationsAdminRequest\x1a\x30.core.mgmt.v1beta.ListOrganizationsAdminResponse\"\x00\x12w\n\x14GetOrganizationAdmin\x12-.core.mgmt.v1beta.GetOrganizationAdminRequest\x1a..core.mgmt.v1beta.GetOrganizationAdminResponse\"\x00\x12\x80\x01\n\x17LookUpOrganizationAdmin\x12\x30.core.mgmt.v1beta.LookUpOrganizationAdminRequest\x1a\x31.core.mgmt.v1beta.LookUpOrganizationAdminResponse\"\x00\x12\x83\x01\n\x18GetUserSubscriptionAdmin\x12\x31.core.mgmt.v1beta.GetUserSubscriptionAdminRequest\x1a\x32.core.mgmt.v1beta.GetUserSubscriptionAdminResponse\"\x00\x12\x9b\x01\n GetOrganizationSubscriptionAdmin\x12\x39.core.mgmt.v1beta.GetOrganizationSubscriptionAdminRequest\x1a:.core.mgmt.v1beta.GetOrganizationSubscriptionAdminResponse\"\x00\x12t\n\x13SubtractCreditAdmin\x12,.core.mgmt.v1beta.SubtractCreditAdminRequest\x1a-.core.mgmt.v1beta.SubtractCreditAdminResponse\"\x00\x12\x80\x01\n\x17GetRemainingCreditAdmin\x12\x30.core.mgmt.v1beta.GetRemainingCreditAdminRequest\x1a\x31.core.mgmt.v1beta.GetRemainingCreditAdminResponse\"\x00\x12t\n\x13\x43heckNamespaceAdmin\x12,.core.mgmt.v1beta.CheckNamespaceAdminRequest\x1a-.core.mgmt.v1beta.CheckNamespaceAdminResponse\"\x00\x12\x83\x01\n\x18\x43heckNamespaceByUIDAdmin\x12\x31.core.mgmt.v1beta.CheckNamespaceByUIDAdminRequest\x1a\x32.core.mgmt.v1beta.CheckNamespaceByUIDAdminResponse\"\x00\x42\xd0\x01\n\x14\x63om.core.mgmt.v1betaB\x17MgmtPrivateServiceProtoP\x01Z=github.com/instill-ai/protogen-go/core/mgmt/v1beta;mgmtv1beta\xa2\x02\x03\x43MX\xaa\x02\x10\x43ore.Mgmt.V1beta\xca\x02\x10\x43ore\\Mgmt\\V1beta\xe2\x02\x1c\x43ore\\Mgmt\\V1beta\\GPBMetadata\xea\x02\x12\x43ore::Mgmt::V1betab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'core.mgmt.v1beta.mgmt_private_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.core.mgmt.v1betaB\027MgmtPrivateServiceProtoP\001Z=github.com/instill-ai/protogen-go/core/mgmt/v1beta;mgmtv1beta\242\002\003CMX\252\002\020Core.Mgmt.V1beta\312\002\020Core\\Mgmt\\V1beta\342\002\034Core\\Mgmt\\V1beta\\GPBMetadata\352\002\022Core::Mgmt::V1beta'
  _globals['_MGMTPRIVATESERVICE']._serialized_start=95
  _globals['_MGMTPRIVATESERVICE']._serialized_end=1593
# @@protoc_insertion_point(module_scope)
